<?php
/**
 * SlideDeck for WordPress - Slider Widget
 * 
 * Create SlideDecks on your WordPress blogging platform. Manage SlideDeck 
 * content and insert them into templates and posts.
 * 
 * @package SlideDeck
 * 
 * @global    object    $wpdb
 * 
 * @author digital-telepathy
 * @version 1.2.1
 */
/*
Plugin Name: SlideDeck for WordPress - Slider Widget
Plugin URI: http://www.slidedeck.com/wordpress
Description: Create SlideDecks on your Wordpress blogging platform and insert them into templates and posts. Get started creating SlideDecks from the new SlideDeck menu in the left hand navigation. <a href="http://www.slidedeck.com/pricing" target="_blank">Upgrade to SlideDeck Pro!</a> | <a href="admin.php?page=slidedeck.php">Manage SlideDecks</a> | <a href="admin.php?page=slidedeck.php/slidedeck_add_new">Add New SlideDeck</a> | <a href="admin.php?page=slidedeck.php/slidedeck_dynamic">Add New Smart SlideDeck</a>
Version: 1.2.1
Author: digital-telepathy
Author URI: http://www.dtelepathy.com
License: GPL2

Copyright 2010 digital-telepathy  (email : support@digital-telepathy.com)

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/



// Adding $wpdb here because it is not loaded automatically on activation,
// causing prefixing to fail on database table names
if ( !isset( $wpdb ) && empty( $wpdb ) ) {
    global $wpdb;
}
define( 'SLIDEDECKS_TABLE',                $wpdb->prefix . "slidedecks" );
define( 'SLIDEDECKS_SLIDES_TABLE',        $wpdb->prefix . "slidedecks_slides" );
define( 'SLIDEDECK_VERSION',            '1.2.0' );
define( 'SLIDEDECK_DB_VERSION',            1.5 );
define( 'SLIDEDECK_EXCERPT_LENGTH_WITH_IMAGE', 30 );
define( 'SLIDEDECK_EXCERPT_LENGTH_WITHOUT_IMAGE', 100 );

/**
 * Installation routine to run upon activation. Function creates two databases,
 * one for storing SlideDeck entries and the other for storing SlideDeck slide
 * content.
 */
if ( function_exists( 'register_activation_hook' ) ) {
    register_activation_hook( __FILE__, 'slidedeck_install' );
}
function slidedeck_install() {
    global $wpdb;
    
    if ( $wpdb->get_var( "SHOW TABLES LIKE '" . SLIDEDECKS_TABLE . "'" ) != SLIDEDECKS_TABLE ) {
        if ( !function_exists( 'dbDelta' ) ) {
            require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
        }
        
        // Setup database to store SlideDeck entries and per-deck configurations
        $SQL = "CREATE TABLE IF NOT EXISTS " . SLIDEDECKS_TABLE . " (
                id INT NOT NULL AUTO_INCREMENT,
                dynamic BOOL NOT NULL DEFAULT '0',
                title VARCHAR( 255 ) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL,
                gallery_id BIGINT( 20 ) UNSIGNED NOT NULL,
                dynamic_options LONGTEXT CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL,
                slidedeck_options LONGTEXT CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL,
                skin VARCHAR(64) NOT NULL DEFAULT 'default',
                created_at DATETIME NOT NULL,
                updated_at TIMESTAMP ON UPDATE CURRENT_TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP,
                PRIMARY KEY  ( id )
                ) CHARACTER SET utf8 COLLATE utf8_general_ci;";
        dbDelta( $SQL );
        
        // Setup database to store slides relationaly associated with SlideDecks
        if ( $wpdb->get_var( "SHOW TABLES LIKE '" . SLIDEDECKS_SLIDES_TABLE . "'" ) != SLIDEDECKS_SLIDES_TABLE ) {
            $SQL = "CREATE TABLE IF NOT EXISTS " . SLIDEDECKS_SLIDES_TABLE . " (
                    id INT NOT NULL AUTO_INCREMENT,
                    slidedeck_id INT NOT NULL,
                    title VARCHAR( 255 ) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL,
                    content LONGTEXT CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL,
                    slide_order TINYINT NOT NULL,
                    created_at DATETIME NOT NULL,
                    updated_at TIMESTAMP ON UPDATE CURRENT_TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP,
                    PRIMARY KEY  ( id ),
                    KEY ( slidedeck_id )
                    ) CHARACTER SET utf8 COLLATE utf8_general_ci;";
            dbDelta( $SQL );
        }
        
        // Store currently installed version in database for future upgrade comparison
        add_option( 'slidedeck_db_version', SLIDEDECK_DB_VERSION );
    } else {
        $current_db_version = floatval( get_option( 'slidedeck_db_version' ) );
        if ( $current_db_version < SLIDEDECK_DB_VERSION ) {
            
            if ( $current_db_version < 1.2 ) {
                if ( !$wpdb->get_var( "SHOW COLUMNS FROM " . SLIDEDECKS_TABLE . " LIKE 'gallery_id';" ) ) {
                    $wpdb->query( "ALTER TABLE " . SLIDEDECKS_TABLE . " ADD gallery_id BIGINT( 20 ) UNSIGNED NOT NULL AFTER start;" );
                }
            }
            
            if ( $current_db_version < 1.3 ) {
                $wpdb->query( "ALTER TABLE " . SLIDEDECKS_TABLE . " CHANGE  title title VARCHAR( 255 ) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL" );
                $wpdb->query( "ALTER TABLE " . SLIDEDECKS_SLIDES_TABLE . " CHANGE  title title VARCHAR( 255 ) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL" );
                $wpdb->query( "ALTER TABLE " . SLIDEDECKS_SLIDES_TABLE . " CHANGE  content content LONGTEXT CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL" );
            }
            
            if ( $current_db_version < 1.4 ) {
                $wpdb->query( "ALTER TABLE " . SLIDEDECKS_TABLE . " DEFAULT CHARACTER SET utf8 COLLATE utf8_general_ci" );
                $wpdb->query( "ALTER TABLE " . SLIDEDECKS_SLIDES_TABLE . " DEFAULT CHARACTER SET utf8 COLLATE utf8_general_ci" );
            }
    
            if ( $current_db_version < 1.5 ) {
                if ( !$wpdb->get_var( "SHOW COLUMNS FROM " . SLIDEDECKS_TABLE . " LIKE 'dynamic';" ) ) {
                    $wpdb->query( "ALTER TABLE " . SLIDEDECKS_TABLE . " ADD dynamic BOOL NOT NULL DEFAULT '0' AFTER id;" );
                    $wpdb->query( "ALTER TABLE " . SLIDEDECKS_TABLE . " ADD dynamic_options LONGTEXT CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL AFTER gallery_id;" );
                    $wpdb->query( "ALTER TABLE " . SLIDEDECKS_TABLE . " ADD slidedeck_options LONGTEXT CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL AFTER dynamic_options;" );
                    $wpdb->query( "ALTER TABLE " . SLIDEDECKS_TABLE . " ADD skin VARCHAR(64) NOT NULL DEFAULT 'default' AFTER slidedeck_options;" );
                    
                    $slidedecks = $wpdb->get_results( 'SELECT id, speed, transition, start FROM ' . SLIDEDECKS_TABLE, ARRAY_A );
                    foreach ( $slidedecks as $slidedeck ) {
                        $slidedeck_options = array(
                            'speed' => $slidedeck['speed'],
                            'start' => $slidedeck['start']
                        );
                        $wpdb->update( SLIDEDECKS_TABLE, array(
                            'slidedeck_options' => serialize( $slidedeck_options )
                        ), array(
                            'id' => $slidedeck['id']
                        ) );
                    }
                    
                    $wpdb->query( "ALTER TABLE " . SLIDEDECKS_TABLE . " DROP speed" );
                    $wpdb->query( "ALTER TABLE " . SLIDEDECKS_TABLE . " DROP transition" );
                    $wpdb->query( "ALTER TABLE " . SLIDEDECKS_TABLE . " DROP start" );
                }
            }
    
            update_option( 'slidedeck_db_version', SLIDEDECK_DB_VERSION );
        }
    }
}



// Setup primary sidebar navigaiton
function slidedeck_menu() {
    add_menu_page( 'SlideDeck', 'SlideDeck', 2, basename( __FILE__ ), 'slidedeck_edit', slidedeck_url( '/images/icon.png' ) );
    add_submenu_page( basename( __FILE__ ), 'Edit SlideDeck', 'Edit', 2, basename( __FILE__ ), 'slidedeck_edit' );
    add_submenu_page( basename( __FILE__ ), 'Add SlideDeck', 'Add New', 2, basename( __FILE__ ) . '/slidedeck_add_new', 'slidedeck_add_new' );
    add_submenu_page( basename( __FILE__ ), 'Add Smart SlideDeck', 'Add Smart SlideDeck', 2, basename( __FILE__ ) . '/slidedeck_dynamic', 'slidedeck_dynamic' );
    add_submenu_page( basename( __FILE__ ), 'Buy SlideDeck Pro', 'Buy SlideDeck Pro', 2, basename( __FILE__ ) . '/upgrade', 'slidedeck_upgrade' );
}
// Load necessary JavaScript and CSS in admin panel ( where appropriate )
function slidedeck_admin_init() {
    // Make exclusion case to only load JavaScript and CSS on the SlideDeck relevant admin pages
    if ( slidedeck_is_plugin() ) {
        if( $_GET['page'] == basename( __FILE__ ) . "/upgrade" ) {
            wp_redirect( "http://www.slidedeck.com/pricing" );
        }
        wp_register_style( 'slidedeck-admin-css', slidedeck_url( '/slidedeck-admin.css' ), array(), SLIDEDECK_VERSION, "screen" );
        wp_register_script( 'slidedeck-admin-js', slidedeck_url( '/slidedeck-admin.js' ), array( 'jquery' ), SLIDEDECK_VERSION );

        wp_enqueue_style( 'slidedeck-admin-css' );
        wp_enqueue_style( 'thickbox' );

        wp_enqueue_script( 'jquery-ui-core' );
        wp_enqueue_script( 'jquery-ui-sortable' );
        wp_enqueue_script( 'thickbox' );
        wp_enqueue_script( 'editor' );
        
        global $slidedeck_skin_included;
        $skins = slidedeck_get_skins();
        foreach( (array) $skins as $skin => $meta ) {
            $slidedeck_skin_included[$skin] = true;
        }
        
        // Process submission to save the SlideDeck
        if( isset( $_POST ) && !empty( $_POST ) ) {
            $slidedeck = slidedeck_save();    // Save SlideDeck
            
            switch( $_POST['action'] ) {
                case "edit":
                    $message_id = 1;
                break;
                
                case "create":
                    $message_id = 2;
                    if( $_POST['dynamic'] == 1 ) {
                        $message_id = 3;
                    }
                break;
            }
            
            wp_redirect( slidedeck_action( $slidedeck['dynamic'] == '1' ? '/slidedeck_dynamic' : '' ) . '&action=edit&id=' . $slidedeck['id'] . '&message=' . $message_id );
        }
    }
}
function slidedeck_admin_head() {
    wp_print_scripts( 'jquery' );

    wp_enqueue_script( 'slidedeck-admin-js' );
    
    if( slidedeck_is_plugin() ) {
        wp_tiny_mce( false, array( 'editor_selector' => 'slide-content' ) );
        slidedeck_print_styles();
    }
}
// Null function for upgrade
function slidedeck_upgrade() {
    return false;
}

/**
 * Update the tinyMCE default embed options, but only on the SlideDeck plugin pages
 * 
 * @param object $initArray
 * @return $initArray
 */
function slidedeck_change_mce_options( $initArray ) {
    if ( slidedeck_is_plugin() ) {
        $initArray['editor_selector'] = 'slide-content';
        $initArray['mode'] = 'specific_textareas';
        $initArray['theme_advanced_buttons1'] = str_replace( ',wp_more', '', $initArray['theme_advanced_buttons1'] );
    }

    return $initArray;
}

/**
 * Convenience method to determine if we are viewing a SlideDeck plugin page
 * 
 * @return boolean
 */
function slidedeck_is_plugin() {
    return (boolean) ( ( "admin.php" == basename( $_SERVER['PHP_SELF'] ) ) && ( strpos( $_GET['page'], basename( __FILE__ ) ) !== false ) );
}


/**
 * Get the root relative server path to the specified plugin file
 * 
 * @param string $str [optional] The path to a file relative to this main plugin file
 * @return string The absolute server path to the file
 */
function slidedeck_dir( $str="" ) {
    $path =  WP_PLUGIN_DIR . "/" . plugin_basename( dirname( __FILE__ ) );
    
    if ( isset( $str ) && !empty( $str ) ) {
        $sep = "/" == substr( $str, 0, 1 ) ? "" : "/";
        return $path . $str;
    } else {
        return WP_PLUGIN_DIR;
    }
}
/**
 * Get the URL for the specified plugin file
 * 
 * @param object $str [optional] The path to the file relative to this main plugin file
 * @return string The absolute URL to the file specified
 * @uses slidedeck_dir()
 */
function slidedeck_url( $str="" ) {
    $path = WP_PLUGIN_URL . "/" . plugin_basename( dirname( __FILE__ ) );
    
    if ( isset( $str ) && !empty( $str ) ) {
        $sep = "/" == substr( $str, 0, 1 ) ? "" : "/";
        return $path . $sep . $str;
    } else {
        return $path;
    }
}
/**
 * Get the URL for the specified plugin action
 * 
 * @param object $str [optional] Expects the handle passed in the menu definition
 * @return The absolute URL to the plugin action specified
 * @uses get_option()
 */
function slidedeck_action( $str=null ) {
    $path = get_bloginfo( 'wpurl' ) . "/wp-admin/admin.php?page=" . basename( __FILE__ );
    
    if ( isset( $str ) && !empty( $str ) ) {
        return $path . $str;
    } else {
        return $path;
    }
}
/**
 * Sanitize data using wp_kses() method
 * 
 * @param str $str Data to sanitize for storage
 * @return str Sanitized version of $str
 * @uses wp_kses()
 */
function slidedeck_sanitize( $str="" ) {
    if ( !function_exists( 'wp_kses' ) ) {
        require_once( ABSPATH . 'wp-includes/kses.php' );
    }
    global $allowedposttags;
    global $allowedprotocols;
    
    if ( is_string( $str ) ) {
        $str = htmlentities( stripslashes( $str ), ENT_QUOTES, 'UTF-8' );
    }
    
    $str = wp_kses( $str, $allowedposttags, $allowedprotocols );
    
    return $str;
}
/**
 * Get meta data for available skins
 * 
 * @param str $type [optional] Expects a string specifying the type of skin to filter to. Default is all skins in the skins folder.
 * @return arr Array of skins with meta data 
 * @uses slidedeck_skin_meta()
 * @uses slidedeck_dir()
 */
function slidedeck_get_skins( $type = 'all' ) {
    $skins = array();
    
    $skin_files = glob( slidedeck_dir( '/skins/*/skin.css' ) );
    foreach ( $skin_files as $skin_file ) {
        if ( is_readable( $skin_file ) ) {
            $skin_meta = slidedeck_skin_meta( $skin_file );
            if ( $type  == 'all' || $skin_meta['meta']['Skin Type'] == $type ) {
                $skins[$skin_meta['slug']] = $skin_meta;
            }
        }
    }
    
    return $skins;
}
/**
 * Get meta data for a specific skin. Expects skin's slug
 * 
 * @param str $name [optional] Slug for skin being looked up, defaults to "default"
 * @return arr Array of skin meta data 
 * @uses slidedeck_skin_meta()
 * @uses slidedeck_dir()
 */
function slidedeck_get_skin( $name = 'default' ) {
    $skin_file = glob( slidedeck_dir( '/skins/' . $name . '/skin.css' ), GLOB_BRACE );
    
    $skin = slidedeck_skin_meta( $skin_file[0] );
    
    return $skin;
}
/**
 * Process skin meta data from a skin file. Used by slidedeck_get_skin and slidedeck_get_skins
 * 
 * @param object $skin_file
 * @return arr Skin meta data
 * @uses slidedeck_url()
 */
function slidedeck_skin_meta( $skin_file ) {
    $skin_data = file_get_contents( $skin_file );
    $skin_folder = dirname( $skin_file );
    $skin_slug = end( explode( "/", $skin_folder ) );
    
    $meta_raw = substr( $skin_data, strpos( $skin_data, "/*" ) + 2 );
    $meta_raw = trim( substr( $meta_raw, 0, strpos( $meta_raw, "*/" ) ) );
    
    if ( !empty( $meta_raw ) ) {
        $skin_meta = array();
        
        foreach ( explode( "\n", $meta_raw ) as $row ) {
            $key_val = explode( ":", $row );
            $skin_meta[trim( $key_val[0] )] = trim( $key_val[1] );
        }
        
        $skin = array(
            'url' => slidedeck_url( substr( $skin_file, strpos( $skin_file, "/skins" ) ) ),
            'thumbnail' => slidedeck_url( substr( $skin_folder, strpos( $skin_folder, "/skins" ) ) . "/thumbnail.png" ),
            'slug' => $skin_slug,
            'meta' => $skin_meta
        );
        if( file_exists( $skin_folder . "/skin.ie.css" ) ) {
            $skin['ie_url'] = slidedeck_url( substr( $skin_folder, strpos( $skin_folder, "/skins" ) ) . "/skin.ie.css" );
        }
        if( file_exists( $skin_folder . "/skin.ie7.css" ) ) {
            $skin['ie7_url'] = slidedeck_url( substr( $skin_folder, strpos( $skin_folder, "/skins" ) ) . "/skin.ie7.css" );
        }
        if( file_exists( $skin_folder . "/skin.ie8.css" ) ) {
            $skin['ie8_url'] = slidedeck_url( substr( $skin_folder, strpos( $skin_folder, "/skins" ) ) . "/skin.ie8.css" );
        }
        if ( file_exists( $skin_folder . "/skin.js" ) ) {
            $skin['script_url'] = slidedeck_url( substr( $skin_folder, strpos( $skin_folder, "/skins" ) ) . "/skin.js" );
        }
        if ( file_exists( $skin_folder . "/templates" ) ) {
            $skin['templates'] = array();
            
            foreach ( glob( $skin_folder . '/templates/*.thtml' ) as $template ) {
                $template_data = file_get_contents( $template );
                $template_raw = substr( $template_data, strpos( $template_data, "/*" ) + 4 );
                $template_raw = trim( substr( $template_raw, 0, strpos( $template_raw, "*/" ) ) );
                
                $template_meta = array();
                $template_meta['slug'] = str_replace( ".thtml", "", basename( $template ) );
                $template_meta['file'] = $template;
                foreach ( explode( "\n", $template_raw ) as $row ) {
                    $key_val = explode( ":", $row );
                    $template_meta[trim( $key_val[0] )] = trim( $key_val[1] );
                }
                
                $skin['templates'][$template_meta['slug']] = $template_meta;
            }
        }
    }
    
    return $skin;
}
/**
 * Compile HTML for skin CSS tags
 * 
 * @param object $skin
 * @return string HTML markup of skin CSS tags
 */
function slidedeck_get_skin_css( $skin ) {
    $version = isset( $skin['meta']['Version'] ) && !empty( $skin['meta']['Version'] ) ? $skin['meta']['Version'] : SLIDEDECK_VERSION;
    
    $skin_css_tags = '<link rel="stylesheet" type="text/css" href="' . $skin['url'] . '?v=' . $version . '" media="screen" />';
    if( isset( $skin['ie_url'] ) && !empty( $skin['ie_url'] ) ) {
        $skin_css_tags .= '<!--[if IE]><link rel="stylesheet" type="text/css" href="' . $skin['ie_url'] . '?v=' . $version . '" media="screen" /><![endif]-->';
    }
    if( isset( $skin['ie7_url'] ) && !empty( $skin['ie7_url'] ) ) {
        $skin_css_tags .= '<!--[if lte IE 7]><link rel="stylesheet" type="text/css" href="' . $skin['ie7_url'] . '?v=' . $version . '" media="screen" /><![endif]-->';
    }
    if( isset( $skin['ie8_url'] ) && !empty( $skin['ie8_url'] ) ) {
        $skin_css_tags .= '<!--[if lte IE 8]><link rel="stylesheet" type="text/css" href="' . $skin['ie8_url'] . '?v=' . $version . '" media="screen" /><![endif]-->';
    }
    
    return $skin_css_tags;
}
/**
 * Convenience method for loading SlideDecks
 * 
 * @param int $slidedeck_id ID of the SlideDeck to retrieve
 * @return arr Single SlideDeck is conditions were passed, all SlideDecks if none were passed
 * @uses $wpdb->get_results()
 * @uses $wpdb->get_row()
 */
function slidedeck_load( $slidedeck_id = null ) {
    global $wpdb;
    
    $SQL = "SELECT id,dynamic,title,gallery_id,dynamic_options,slidedeck_options,skin,created_at,updated_at FROM " . SLIDEDECKS_TABLE;
    
    // No ID passed, get all SlideDecks
    if ( !isset( $slidedeck_id ) ) {
        $slidedeck = $wpdb->get_results( $SQL . " ORDER BY title ASC;", ARRAY_A );
    }
    // ID passed, process as multiple result request or singular
    else {
        if( is_array( $slidedeck_id ) ) {
            $sep = " WHERE";
            foreach( $slidedeck_id as $slidedeck_id_single ) {
                $SQL .= $sep . " id=" . $slidedeck_id_single;
                $sep = " OR";
            }
            $slidedeck = $wpdb->get_results( $wpdb->prepare( $SQL, $slidedeck_id ), ARRAY_A );
        } else {
            $SQL .= " WHERE id=%d";
            $slidedeck = $wpdb->get_row( $wpdb->prepare( "SELECT id,dynamic,title,gallery_id,dynamic_options,slidedeck_options,skin,created_at,updated_at FROM " . SLIDEDECKS_TABLE . " WHERE id=%d", $slidedeck_id ), ARRAY_A );
        }
    }
    
    if( !empty( $slidedeck ) ) {
        // IF no $slidedeck_id was passed or, multiple were requested, process $slidedeck as an array
        if( !isset( $slidedeck_id ) || is_array( $slidedeck_id ) ) {
            for ( $i = 0; $i < count( $slidedeck ); $i++ ) {
                $slidedeck[$i]['slidedeck_options'] = unserialize( $slidedeck[$i]['slidedeck_options'] );
                $slidedeck[$i]['dynamic_options'] = unserialize( $slidedeck[$i]['dynamic_options'] );
            }
        }
        // Process $slidedeck as a singular result
        else {
            $slidedeck['slidedeck_options'] = unserialize( $slidedeck['slidedeck_options'] );
            $slidedeck['dynamic_options'] = unserialize( $slidedeck['dynamic_options'] );
        }
    } else {
        $slidedeck = false;
    }
    
    return $slidedeck;
}
/**
 * Get SlideDeck options for a SlideDeck. Used to make adding new SlideDeck JavaScript library
 * features easier (database values do not need to be pre-defined).
 * 
 * @param object $slidedeck SlideDeck object
 * @param string $option Option to return
 * @return $slidedeck['slidedeck_options'] value or default value
 */
function slidedeck_get_option( $slidedeck, $option ) {
    // Defaults for the $slidedeck['slidedeck_options'] options array
    $defaults = array( 
        'speed' => 500,
        'start' => 1,
        'autoPlay' => 'false',
        'autoPlayInterval' => 5000,
        'cycle' => 'false',
        'hideSpines' => 'false',
        'transition' => 'swing'
    );
    
    return ( isset( $slidedeck['slidedeck_options'][$option] ) ? $slidedeck['slidedeck_options'][$option] : $defaults[$option] );
}

/**
 * Get Dynamic SlideDeck options for a SlideDeck. Used to make adding new SlideDeck JavaScript library
 * features easier (database values do not need to be pre-defined).
 * 
 * @param object $slidedeck SlideDeck object
 * @param string $option Option to return
 * @return $slidedeck['dynamic_options'] value or default value
 */
function slidedeck_get_dynamic_option( $slidedeck, $option ) {
    // Defaults for the $slidedeck['slidedeck_options'] options array
    $defaults = array( 
        'type' => 'recent',
        'filter_by_category' => 0,
        'filter_categories' => array(),
        'navigation_type' => 'simple-dots',
        'total' => 5,
        'cache_minutes' => 30,
        'image_source' => 'content',
        'excerpt_length_with_image' => SLIDEDECK_EXCERPT_LENGTH_WITH_IMAGE,
        'excerpt_length_without_image' => SLIDEDECK_EXCERPT_LENGTH_WITHOUT_IMAGE,
        'feed_url' => "",
        'validate_images' => 0
    );
    
    return ( isset( $slidedeck['dynamic_options'][$option] ) ? $slidedeck['dynamic_options'][$option] : $defaults[$option] );
}

function slidedeck_show_message() {
    if( !isset( $_GET['message'] ) ) {
        return false;
    }
    
    $messages = array(
        '<strong>SlideDeck updated!</strong> Now <a href="' . get_bloginfo( 'wpurl' ) . '/wp-admin/edit.php">insert your SlideDeck into a post</a> or use the <a href="#get-slidedeck-template-snippet"><em>Theme Code Snippet</em></a> to place it in your theme.',
        '<strong>New SlideDeck created!</strong> Now <a href="' . get_bloginfo( 'wpurl' ) . '/wp-admin/edit.php">insert your SlideDeck into a post</a> or use the <a href="#get-slidedeck-template-snippet"><em>Theme Code Snippet</em></a> to place it in your theme.',
        '<strong>New Smart SlideDeck created!</strong> Now use the <a href="#get-slidedeck-template-snippet"><em>Theme Code Snippet</em></a> to place it in your templates.',
        'SlideDeck deleted!'
    );
    
    $message_id = ( intval( $_GET['message'] ) - 1 );
    
    echo '<div id="message" class="updated fade below-h2" style="background-color: rgb(255,251,204);"><p>' . $messages[$message_id] . '</p></div>';
}

// The processor for the "Add New" menu item 
function slidedeck_add_new() {
    global $wpdb;
    
    $form_action = "create";    // Set the form action ( referenced when saving the SlideDeck and for interface appearance )
    $default_slide_amount = 3;    // Set the default amount of slides to start with
    
    $skins = slidedeck_get_skins();
    
    // Set the default SlideDeck settings
    $slidedeck = array( 
        'title' => "",
        'slidedeck_options' => array(),
        'gallery_id' => time(),
        "skin" => "default"
     );

    // Populate the default slide values
    $slides = array();
    $slide_names = array();
    for ( $i = 1; $i <= $default_slide_amount; $i++ ) {
        $slides[] = array( 
            'title' => "",
            'content' => "",
            'slide_order' => $i,
            'gallery_id' => $slidedeck['gallery_id']
         );
    }
    
    // Render the editor form
    include( slidedeck_dir( '/views/edit-form.php' ) );
}



// The processor for the "Add New Dynamic" menu item
function slidedeck_dynamic() {
    global $wpdb, $wp_version;
    
    $form_action = "create";
    if ( isset( $_GET['action'] ) && !empty( $_GET['action'] ) ) {
        $form_action = $_GET['action'];
    }
    
    $default_slide_amount = 5;
    $skins = slidedeck_get_skins( 'dynamic' );
    $categories = get_categories( array(
        'type' => 'post',
        'orderby' => 'name',
        'order' => 'ASC',
        'hide_empty' => false
    ) );
    
    $slidedeck = array( 
        'title' => "",
        'slidedeck_options' => array(
            'speed' => 500,
            'start' => 1,
            'autoPlay' => 'true',
            'autoPlayInterval' => 5000,
            'cycle' => 'true',
            'hideSpines' => 'true'
        ),
        'dynamic_options' => array(
            'type' => 'recent',
            'filter_by_category' => 0,
            'filter_categories' => array(),
            'navigation_type' => 'simple-dots',
            'total' => 5,
            'cache_minutes' => 30,
            'image_source' => 'content',
            'excerpt_length_with_image' => SLIDEDECK_EXCERPT_LENGTH_WITH_IMAGE,
            'excerpt_length_without_image' => SLIDEDECK_EXCERPT_LENGTH_WITHOUT_IMAGE
        ),
        'skin' => 'dark',
        'gallery_id' => time(),
        'dynamic' => 1
    );
    if ( isset( $_GET['id'] ) && !empty( $_GET['id'] ) ) {
        $slidedeck_id = intval( $_GET['id'] );
        
        $slidedeck = slidedeck_load( $slidedeck_id );
    }
    
    include( slidedeck_dir( '/views/add-edit-dynamic.php' ) );
}



// The processor for the "Edit" and "Overview" menu items
function slidedeck_edit() {
    global $wpdb;
    
    $action = "overview";    // Set the default action to overview to display a list of SlideDecks
    // Override the default action if one has been specified
    if ( isset( $_GET['action'] ) && !empty( $_GET['action'] ) ) {
        $action = $_GET['action'];
    }
    
    $skins = slidedeck_get_skins();
    
    // Change processing based off of action specified
    switch ( $action ) {
        case "edit":
            $form_action = "edit";
            $slidedeck_id = $_GET['id'];
            
            // Get the SlideDeck
            $slidedeck = slidedeck_load( $slidedeck_id );

            // Get the SlideDeck's slides
            $slides = $wpdb->get_results( $wpdb->prepare( "SELECT id,slidedeck_id,title,content,slide_order,created_at,updated_at FROM " . SLIDEDECKS_SLIDES_TABLE . " WHERE slidedeck_id = %d ORDER BY slide_order ASC;", $slidedeck_id ), ARRAY_A );
            
            // Accommodate for entries without a gallery_id
            if ( $slidedeck['gallery_id'] == '0' ) {
                $slidedeck['gallery_id'] = time();
            }

            // Assign the SlideDeck's gallery_id to each slide
            for ( $i=0; $i<count( $slides ); $i++ ) {
                $slides[$i]['gallery_id'] = $slidedeck['gallery_id'];
            }
            
            // Render the editor form
            include( slidedeck_dir( '/views/edit-form.php' ) );
        break;
        
        case "delete":
            if( check_admin_referer( 'slidedeck-delete' ) ) {
                // Delete the SlideDeck entry
                $wpdb->query( $wpdb->prepare( "DELETE FROM " . SLIDEDECKS_TABLE . " WHERE id = %d;", $_GET['id'] ) );
                // Delete the SlideDeck slides associated with the SlideDeck entry
                $wpdb->query( $wpdb->prepare( "DELETE FROM " . SLIDEDECKS_SLIDES_TABLE . " WHERE slidedeck_id = %d;", $_GET['id'] ) );
            } else {
                return false;
            }

        default:
            // Get a list of all SlideDecks in the system
            $slidedecks = slidedeck_load();
            
            // Render the overview list
            include( slidedeck_dir( '/views/overview.php' ) );
        break;
    }
}



// Processor to save SlideDecks - shared by create and edit form submissions
function slidedeck_save() {
    global $wpdb;
    
    $action = $_POST['action'];

    if( !wp_verify_nonce( $_POST['slidedeck-' . $action . '_wpnonce'], 'slidedeck-for-wordpress' ) ) {
        return false;
    }

    $params = array();
    foreach ( (array) $_POST as $key => $val ) {
        if ( is_string( $val ) ) {
            $params[$key] = slidedeck_sanitize( $val );
        } elseif ( is_array( $val ) ) {
            foreach ( (array) $val as $key1 => $val1 ) {
                if ( is_array( $val1 ) ) {
                    $sub_arr = array();
                    foreach ( (array) $val1 as $key2 => $val2 ) {
                        $sanitized = $val2;
                        switch ( $key2 ) {
                            case "title":
                            case "content":
                                $sanitized = slidedeck_sanitize( $val2 );
                            break;
                        }
                        $sub_arr[$key2] = $sanitized;
                    }
                    $params[$key][$key1] = $sub_arr;
                } else {
                    $params[$key][$key1] = slidedeck_sanitize( $val1 );
                }
            }
        }
    }
    
    // Begin building formatted array of data to be saved
    $slidedeck_params = array( 
        'title' => $params['title'],
        'dynamic' => $params['dynamic'],
        'gallery_id' => $params['gallery_id']
    );
    
    if ( $params['dynamic'] == '1' ) {
        if ( !isset( $params['dynamic_options']['filter_by_category'] ) || !isset( $params['dynamic_options']['filter_categories'] ) ) {
            $params['dynamic_options']['filter_by_category'] = '0';
            $params['dynamic_options']['filter_categories'] = array();
        }
        $slidedeck_params['dynamic_options'] = serialize( $params['dynamic_options'] );
    }

    $slidedeck_params['skin'] = $params['skin'];

    if ( !isset( $params['slidedeck_options']['autoPlay'] ) ) {
        $params['slidedeck_options']['autoPlay'] = '0';
    }
    if ( !isset( $params['slidedeck_options']['cycle'] ) ) {
        $params['slidedeck_options']['cycle'] = 'false';
    }
    if ( !isset( $params['slidedeck_options']['hideSpines'] ) ) {
        $params['slidedeck_options']['hideSpines'] = 'false';
    }
    $params['slidedeck_options']['autoPlayInterval'] = $params['slidedeck_options']['autoPlayInterval'] * 1000;
    $slidedeck_params['slidedeck_options'] = serialize( $params['slidedeck_options'] );
    
    // Process based off of action - is this new or updating an existing entry
    switch ( $action ) {
        case "create":
            $slidedeck_params['created_at'] = date( "Y-m-d H:i:s" );
            
            // Insert a new SlideDeck in the database
            $wpdb->insert( SLIDEDECKS_TABLE, $slidedeck_params );
            
            // Retrieve the last SlideDeck created to get the ID of the new database entry
            $slidedeck_id = $wpdb->insert_id; 
            $slidedeck = slidedeck_load( $slidedeck_id );
            
            // Insert new slide entries linked to the newly created SlideDeck if this is not a dynamic SlideDeck
            if ( $params['dynamic'] != '1' ) {
                foreach ( (array) $params['slide'] as $slide ) {
                    $wpdb->insert( SLIDEDECKS_SLIDES_TABLE, array( 
                        'slidedeck_id' => $slidedeck['id'],
                        'title' => $slide['title'],
                        'content' => $slide['content'],
                        'slide_order' => $slide['slide_order'],
                        'created_at' => $slidedeck_params['created_at']
                    ) );
                }
            }
        break;
        
        case "edit":
        	$slidedeck_id = $params['id'];
            
            if ( $params['dynamic'] != '1' ) {
                // Get the existing slides associated with this SlideDeck
                $slides = $wpdb->get_results( $wpdb->prepare( "SELECT id,slidedeck_id,title,content,slide_order,created_at,updated_at FROM " . SLIDEDECKS_SLIDES_TABLE . " WHERE slidedeck_id = %d;", $_POST['id'] ), ARRAY_A );
                $existing_slides = array();
                $new_slides = array();

                // Create an array of IDs of the current slides associated with this SlideDeck prior to submission
                foreach ( (array) $slides as $slide ) {
                    if( isset( $slide['id'] ) && !empty( $slide['id'] ) ) {
                        $existing_slides[] = $slide['id'];
                    }
                }
                
                foreach ( (array) $params['slide'] as $slide ) {
                    // Compare each submitted slide to see if it exists in the database already
                    if ( isset( $slide['id'] ) && !empty( $slide['id'] ) ) {
                        $slide_id = $slide['id'];

                        // If slide exists, update it
                        $wpdb->update( SLIDEDECKS_SLIDES_TABLE, array( 
                            'slidedeck_id' => $params['id'],
                            'title' => $slide['title'],
                            'content' => $slide['content'],
                            'slide_order' => $slide['slide_order']
                         ), array( 
                            'id' => $slide_id
                         ) );

                        // Add the submitted slide ID to an array to later compare against the previously existing slides
                        $new_slides[] = $slide_id;
                    } else {
                        // If slide does not exist yet, add it to the database
                        $created_at = date( "Y-m-d H:i:s" );
                        $wpdb->insert( SLIDEDECKS_SLIDES_TABLE, array( 
                            'slidedeck_id' => $params['id'],
                            'title' => $slide['title'],
                            'content' => $slide['content'],
                            'slide_order' => $slide['slide_order'],
                            'created_at' => $created_at
                         ) );
                        
                        // Add the ID of the new slide to the comparison array
                        $slide_id = $wpdb->insert_id;
                        $new_slides[] = $slide_id;
                    }
                }
                
                // Compare the array of slides that existed prior to submission to the slides that were submitted
                foreach ( (array) $existing_slides as $slide_id ) {
                    // If the previously existing slide is not found in the submitted slides array, user has deleted it - remove the record from the database
                    if ( !in_array( $slide_id, $new_slides ) ) {
                        $wpdb->query( $wpdb->prepare( "DELETE FROM " . SLIDEDECKS_SLIDES_TABLE ." WHERE id = %d;", $slide_id ) );
                    }
                }
            }
        break;
    }
    
    // Update SlideDeck in the database
    $wpdb->update( SLIDEDECKS_TABLE, $slidedeck_params, array( 
        'id' => $slidedeck_id
    ) );
                
    // Get the record for the submitted SlideDeck to return the newly saved values
    $slidedeck = slidedeck_load( $slidedeck_id );
    
    return $slidedeck;
}



// Setup TinyMCE button
function slidedeck_addbuttons() {
    // Setup the stylesheet to use for the modal window interaction
    wp_register_style( 'slidedeck-ui-styles', slidedeck_url( '/lib/tinymce3/slidedeck-jquery-ui.css' ) );

    // Return false if the user does not have WYSIWYG editing privileges
    if ( !current_user_can( 'edit_posts' ) && !current_user_can( 'edit_pages' ) ) {
        return false;
    }
    
    // Add buttons to TinyMCE editor if user can edit with WYSIWYG editor
    if ( 'true' == get_user_option( 'rich_editing' ) ) {
        add_filter( 'mce_external_plugins', 'slidedeck_add_tinymce_plugin' );
        add_filter( 'mce_buttons', 'slidedeck_register_button' );
    }

    // Only load the necessary scripts if the user is on the post/page editing admin pages
    if ( in_array( basename( $_SERVER['PHP_SELF'] ), array( 'post-new.php', 'page-new.php', 'post.php', 'page.php' ) ) ) {
        wp_enqueue_script( 'jquery-ui-dialog' );
        wp_enqueue_style( 'slidedeck-ui-styles' );
    }
}
// Add the SlideDeck button to the TinyMCE interface
function slidedeck_register_button( $buttons ) {
    array_push( $buttons, "separator", "slidedeck" );
    return $buttons;
}
// Add the SlideDeck TinyMCE plugin to the TinyMCE plugins list
function slidedeck_add_tinymce_plugin( $plugin_array ) {
    if( !slidedeck_is_plugin() ) {
        $plugin_array['slidedeck'] = slidedeck_url( '/lib/tinymce3/editor-plugin.js' );
    }

    return $plugin_array;
}



// Create the modal window dialog box for the TinyMCE plugin
function slidedeck_tinymce_plugin_dialog() {
    // Only load the necessary scripts and render the modal window dialog box if the user is on the post/page editing admin pages
    if ( in_array( basename( $_SERVER['PHP_SELF'] ), array( 'post-new.php', 'page-new.php', 'post.php', 'page.php' ) ) ) {
        global $wpdb;
        
        $slidedecks = $wpdb->get_results( "SELECT * FROM " . SLIDEDECKS_TABLE . " ORDER BY updated_at;", ARRAY_A );
        
        include( slidedeck_dir( '/views/_tinymce-plugin-dialog.php' ) );
    }
}



// Run by the "wp" hook event to determine which SlideDecks are being loaded on this page
function slidedeck_wp_hook() {
    global $posts, $slidedeck_skin_included;
    
    $slidedeck_skin_included['default'] = true;
    
    if( isset( $posts ) && !empty( $posts ) ) {
        $slidedeck_ids = array();
    
        // Process through $posts for the existence of SlideDecks
        foreach( (array) $posts as $post ) {
            $matches = array();
            preg_match_all( '/\[SlideDeck( ([a-zA-Z0-9]+)\=\'([a-zA-Z0-9\%\-_\.]+)\')*\]/', $post->post_content, $matches );
            if( !empty( $matches[0] ) ) {
                foreach( $matches[0] as $match ) {
                    $str = $match;
                    $str_pieces = explode( " ", $str );
                    foreach( $str_pieces as $piece ) {
                        $attrs = explode( "=", $piece );
                        if( $attrs[0] == "id" ) {
                            // Add the ID of this SlideDeck to the ID array for loading
                            $slidedeck_ids[] = intval( str_replace( "'", '', $attrs[1] ) );
                        }
                    }
                }
            }
        }
        // Load SlideDecks used on this URL passing the array of IDs
        $slidedecks = slidedeck_load( $slidedeck_ids );

        // Loop through SlideDecks used on this page and add their skins to the $slidedeck_skin_included array for later use
        foreach( (array) $slidedecks as $slidedeck ) {
            $skin_slug = isset( $slidedeck['skin'] ) && !empty( $slidedeck['skin'] ) ? $slidedeck['skin'] : 'default';
            $slidedeck_skin_included[$skin_slug] = true;
        }
    }
}



// Load the SlideDeck library JavaScript and support files in the public views to render SlideDecks
function slidedeck_print_scripts() {
    global $slidedeck_skin_included;
    
    wp_register_script( 'slidedeck-library-js', slidedeck_url( '/lib/slidedeck.jquery.lite.pack.js' ), array( 'jquery' ), SLIDEDECK_VERSION );

    wp_enqueue_script( 'jquery' );
    wp_enqueue_script( 'slidedeck-library-js' );
    
    foreach( (array) $slidedeck_skin_included as $skin_slug => $val ) {
        $skin = slidedeck_get_skin( $skin_slug );
        if( isset( $skin['script_url'] ) ) {
            wp_register_script( 'slidedeck-skin-js-' . $skin_slug, $skin['script_url'], array( 'jquery', 'slidedeck-library-js' ), SLIDEDECK_VERSION );
            wp_enqueue_script( 'slidedeck-skin-js-' . $skin_slug );
        }
    }
}



// Load SlideDeck support CSS files based off of $slidedeck_skin_included for all skins needed for the page
function slidedeck_print_styles() {
    global $slidedeck_skin_included;
    
    foreach( (array) $slidedeck_skin_included as $skin_slug => $val ) {
        $skin = slidedeck_get_skin( $skin_slug );
        echo slidedeck_get_skin_css( $skin );
    }
}
 


// Process the SlideDeck shortcode
function slidedeck_shortcode( $atts ) {
    extract( shortcode_atts( array(
        'id' => false,
        'width' => '100%',
        'height' => '300px'
    ), $atts ) );
    
    if ( $id !== false ) {
        return slidedeck_process_template( $id, array( 'width' => $width, 'height' => $height ) );
    } else {
        return "";
    }
}

/**
 * SlideDeck markup creator
 * 
 * @param object $id The ID of the SlideDeck to render
 * @param array $styles [optional] The styles to apply to the main SlideDeck tag ( usually just width and height )
 * @return Rendered SlideDeck markup and JavaScript tag to initialize SlideDeck render
 * 
 * @uses slidedeck_load()
 * @uses slidedeck_dir()
 * @uses slidedeck_dynamic_template_content()
 * @uses slidedeck_get_skin()
 * @uses WP_Query
 * @uses get_the_title()
 * @uses get_the_time()
 * @uses human_time_diff()
 * @uses wp_trim_excerpt()
 * @uses get_permalink()
 * @uses get_the_content()
 * @uses get_post_custom_values()
 * @uses $wpdb
 */
function slidedeck_process_template( $id, $styles = array( 'width' => '100%', 'height' => '300px' ) ) {
    global $wpdb, $slidedeck_skin_included, $slidedeck_footer_scripts;
    
    // Lookup the SlideDeck requested
    $slidedeck = slidedeck_load( $id );
    
    if ( isset( $slidedeck ) && !empty( $slidedeck ) ) {
        $is_dynamic = (boolean) $slidedeck['dynamic'];

        if ( $is_dynamic === true ) {
            $styles['width'] = "100%";
            $styles['height'] = "370px";
        }

        $skin = slidedeck_get_skin( ( isset( $slidedeck['skin'] ) && !empty( $slidedeck['skin'] ) ) ? $slidedeck['skin'] : 'default' );
        if( $skin['meta']['Skin Type'] == "fixed" ) {
            $styles['height'] = $skin['meta']['Skin Height'] . "px";
        }
        
        // Setup styles array as inline style string
        $styles_str = "";
        $sep = "";
        foreach ( (array) $styles as $style => $def ) {
            $styles_str.= $sep . $style . ":" . $def;
            $sep = ";";
        }
    
        srand(); // Seed the random number generator
        // Create unique SlideDeck ID for this SlideDeck
        $slidedeck_uid = "SlideDeck_" . rand( 100, 999 ) . "_" . $slidedeck['id'];
        
        if ( $is_dynamic === true ) {
            $args = array(
                'post_type' => 'post',
                'post_status' => 'publish',
                'posts_per_page' => $slidedeck['dynamic_options']['total']
            );
            if ( $slidedeck['dynamic_options']['filter_by_category'] == '1' ) {
                $args['cat'] = implode( ',', $slidedeck['dynamic_options']['filter_categories'] );
            }
            switch ( $slidedeck['dynamic_options']['type'] ) {
                case "recent":
                    $args['orderby'] = 'date';
                    $args['order'] = 'DESC';
                break;
                
                case "featured":
                    $args['meta_key'] = '_slidedeck_post_featured';
                    $args['meta_value'] = '1';
                    $args['orderby'] = 'date';
                    $args['order'] = 'DESC';
                break;
                
                case "popular":
                    $args['orderby'] = '_slidedeck_popularity';
                    $args['order'] = 'DESC';
                break;
            }
            
            $excerpt_length_with_image = SLIDEDECK_EXCERPT_LENGTH_WITH_IMAGE;
            $excerpt_length_without_image = SLIDEDECK_EXCERPT_LENGTH_WITHOUT_IMAGE;

            // Post Based Dynamic SlideDeck.
            $dynamic_posts = new WP_Query( $args );
            
            $slides = array();
            while ( $dynamic_posts->have_posts() ) {
                $dynamic_posts->the_post();
                
                $slide = array();
                
                $slide_nodes = array(
                    'type' => $slidedeck['dynamic_options']['type'],
                    'title' => html_entity_decode( get_the_title(), ENT_QUOTES, 'UTF-8' ),
                    'timestamp' => strtotime( get_the_time( 'c' ) ),
                    'timesince' => "Posted " . human_time_diff( strtotime( get_the_time( 'c' ) ) ) . " ago",
                    'permalink' => get_permalink()
                );
                
                $post_content = get_the_content();
                
                switch( slidedeck_get_dynamic_option( $slidedeck, 'image_source' ) ) {
                    case 'none':
                        $slide_nodes['image'] = null;
                    break;
                    default:
                    case 'content':
                        $slide_nodes['image'] = slidedeck_parse_html_for_images( $post_content, slidedeck_get_dynamic_option( $slidedeck, 'validate_images' ) );
                    break;
                }
                
                
                // Unfortunately we cannot use WP's built in excerpt shortener since
                // we cannot easily access the existing set excerpt length and shouldn't
                // modify the user's perferences here
                $the_excerpt_limit = isset( $slide_nodes['image'] ) ? $excerpt_length_with_image : $excerpt_length_without_image;
                $the_excerpt = strip_shortcodes( get_the_excerpt() );
                
                $slide_nodes['excerpt'] = slidedeck_prepare_excerpt( $the_excerpt, $the_excerpt_limit );
                
                // Process content nodes through template to create slide content
                ob_start();
    
                    foreach ( $slide_nodes as $node => $val ) {
                        $$node = $val;
                    }
    
                    include( $skin['templates']['two_column_copy_image']['file'] );
                    $slide['content'] = ob_get_contents();
    
                    foreach ( $slide_nodes as $node => $val ) {
                        $$node = null;
                    }
    
                ob_end_clean();
                
                // Process slide title with post title or specified slide title
                $spine_title = get_post_meta( get_the_ID(), '_slidedeck_slide_title', true );
                $slide['title'] = !empty( $spine_title ) ? $spine_title :  get_the_title();
                
                $slide['timestamp'] = $slide_nodes['timestamp'];
                
                $slides[] = $slide;
            }
        } else {
            // Get the Slides for the SlideDeck requested
            $slides = $wpdb->get_results( $wpdb->prepare( "SELECT * FROM " . SLIDEDECKS_SLIDES_TABLE . " WHERE slidedeck_id = %d ORDER BY slide_order ASC;", $slidedeck['id'] ), ARRAY_A );
        }
        
        // Generate markup for the template string.
        $template_str = slidedeck_output( $slidedeck, $slidedeck_uid, $slides, $styles_str );
        
        // Build the JavaScript statement.
        $slidedeck_options_json = "{ ";
        $sep = "";
        foreach ( (array) $slidedeck['slidedeck_options'] as $key => $val ) {
            $slidedeck_options_json.= $sep . $key . ": ";

            if ( $val == 'true' || $val == 'false' ) { $slidedeck_options_json.= $val; }
            elseif ( intval( $val ) == $val ) { $slidedeck_options_json.= $val; }
            else { $slidedeck_options_json.= (string) "'{$val}'"; }
            
            $sep = ", ";
        }
        $slidedeck_options_json.= " }";
        
        if ( $is_dynamic === true ) {
            $template_str.= slidedeck_dynamic_template_content( $slidedeck, $slides );
        }
        
        $template_str.= '</div>';

        $slidedeck_footer_scripts .= '<script type="text/javascript">jQuery( \'#' . $slidedeck_uid . '\' ).slidedeck( ' . $slidedeck_options_json . ' );</script><noscript>Powered by <a href="http://www.slidedeck.com" target="_blank">SlideDeck</a></noscript>';
        
        if( !isset( $slidedeck_skin_included[$skin['slug']] ) ) {
            $slidedeck_skin_included[$skin['slug']] = true;

            $skin_css_tags = slidedeck_get_skin_css( $skin );
            $template_str = $skin_css_tags . $template_str;

            if ( isset( $skin['script_url'] ) && !empty( $skin['script_url'] ) ) {
                $slidedeck_footer_scripts .= '<script type="text/javascript" src="' . $skin['script_url'] .'"></script>';
            }
        }
        
    } else {
        $template_str = "";
    }
    
    return $template_str;
}

/**
 * Used for printing out the JavaScript commands to load SlideDecks and appropriately
 * read the DOM for positioning, sizing, dimensions, etc.
 * 
 * @return Echo out the JavaScript tags generated by slidedeck_process_template;
 */
function slidedeck_print_footer_scripts() {
    global $slidedeck_footer_scripts;
    
    echo $slidedeck_footer_scripts;
}



/**
 * Used for trimming HTML down to a certain length
 * 
 * @param str Input HTML
 * @param int Length of words to output
 * @return str Trimmed input HTML
 */
function slidedeck_prepare_excerpt( $input, $limit ) {
    if( $limit > 0 ) {
        $the_excerpt = trim( strip_tags( $input ) );
        $the_excerpt_pieces = explode( " ", $the_excerpt );
        
        // Remove empty values from the array.
        foreach ( $the_excerpt_pieces as $key => $value ) {
          if ( empty( $value ) ) {
            unset( $the_excerpt_pieces[$key] );
          }
        }
        
        // Only return something if there's something to return.
        if( count( $the_excerpt_pieces ) > 0 ) {
            $slidedeck_trimmed_excerpt = implode( " ", array_slice( $the_excerpt_pieces, 0, $limit ) );
            if ( count( $the_excerpt_pieces ) > $limit ) {
                $slidedeck_trimmed_excerpt.="&hellip;";
            }
            return wpautop( html_entity_decode( $slidedeck_trimmed_excerpt, ENT_QUOTES, 'UTF-8' ), false );
        }
    }
}

/**
 * Parses raw HTML and returns an array of images 
 * 
 * @param str Raw HTML to be processed
 * @return arr Array of images found in the passed in HTML
 */
function slidedeck_parse_html_for_images( $html_string, $validate = false ) {
    $html_string = str_replace( array( "\n", "\r" ), array( " ", " " ), $html_string);
    $image_raw = substr( $html_string, strpos( $html_string, '<img ' ) );
    $image_raw = substr( $image_raw, 0, strpos( $image_raw, '>' ) );
    
    $image_strs = array();
    preg_match_all( '/<img( ([a-zA-Z]+)\=\"([a-zA-Z0-9\/\#\&\=\-_\+\%\!\?\:\;\.\(\)\s]*)\")+\s*\/?>/', $html_string, $image_strs );
    
    $images = array();
    if( isset( $image_strs[0] ) && !empty( $image_strs[0] ) ) {
        foreach( (array) $image_strs[0] as $image_str ) {
            $image_attr = array();
            $image_substr = preg_match_all( '/([a-zA-Z]+)\=\"([a-zA-Z0-9\/\#\&\=\-_\+\%\?\:\;\.\(\)\s]+)\"/', $image_str, $image_attr );
            
            if( in_array( 'src', $image_attr[1] ) ) {
                $images[] = array_combine( $image_attr[1], $image_attr[2] );
            }
        }
    }
    
    $output = false;
    $output_set = false;
    if( !empty( $images ) ) {
        $output = $images[0];
        
        // If validation for this SlideDeck is turned on, it will look for images with a width attribute (helps weed out some advertisements)
        if( (boolean) $validate === true ) {
            $output = false;
            foreach( $images as $image ) {
                if( $output_set === false ) {
                    if( isset( $image['width'] ) && !empty( $image['width'] ) ) {
                        $output = $image;
                        $output_set = true;
                    }
                }
            }
        }
    }
    
    return $output;
}

/**
 * Process extra code needed for dynamic SlideDecks
 * 
 * @param object $slidedeck SlidDeck Object with meta information as returned by slidedeck_load()
 * @param object $slides Slides Object with meta information describing each slide
 * @return str HTML markup for extra navigational elements for dynamic SlideDecks
 */
function slidedeck_dynamic_template_content( $slidedeck, $slides ) {
    $template_str = '<a href="#previous" class="sd-node sd-node-nav-link sd-node-previous">Previous</a><a href="#next" class="sd-node sd-node-nav-link sd-node-next">Next</a>';
    $template_str.= '<ul class="sd-node sd-node-nav sd-node-nav-primary sd-node-navigation-type-' . $slidedeck['dynamic_options']['navigation_type'] . ' sd-node-total-' . count( $slides ) . '">';
    
    $i = 1;
    foreach ( (array) $slides as $slide ) {
        $buffer = 0;
        $title_cap = ( 37 + ( ( 5 - $slidedeck['dynamic_options']['total'] ) * 10 ) );
        $title_substr = substr( $slide['title'], 0, $title_cap ) . ( strlen( $slide['title'] ) < $title_cap ? '' : '&hellip;' );
        $template_str.= '<li><a href="#' . $i . '" class="sd-node-nav-link"><span class="sd-node-nav-link-label-date">' . date( 'M j', $slide['timestamp'] ) . '</span><span class="sd-node-nav-link-label-title">' . $title_substr . '</span></a></li>';
        $i++;
    }
    
    $template_str.= '</ul>';
    
    return $template_str;
}

/**
 * SlideDeck markup creator public function. This function can be called from a template or theme
 * to embed a SlideDeck in your layout.
 * 
 * @param object $id The ID of the SlideDeck to render
 * @param array $styles [optional] The styles to apply to the main SlideDeck tag ( usually just width and height )
 * @return Rendered SlideDeck markup and JavaScript tag to initialize SlideDeck render
 * 
 * @uses slidedeck_process_template()
 */
function slidedeck( $id, $styles=array( 'width' => '100%', 'height' => '300px' ) ) {
    // Send the SlideDeck parameters through the output template processor
    $template_str = slidedeck_process_template( $id, $styles );

    echo $template_str;
}


/**
 * Create the markup for a standard SlideDeck
 * 
 * @param array $slidedeck The SlideDeck
 * @param object $slidedeck_uid A Unique identifier for the outputted SlideDeck
 * @param array $slides The array of slides to be rendered
 * @return SlideDeck HTML markup
 * 
 */
function slidedeck_output( $slidedeck, $slidedeck_uid, $slides, $styles_str ) {
    $inc = 1;
    $template_str = '<div class="slidedeck_frame skin-' . ( ( isset( $slidedeck['skin'] ) && !empty( $slidedeck['skin'] ) ) ? $slidedeck['skin'] : 'default' ) . '"><dl id="' . $slidedeck_uid . '" class="slidedeck slidedeck_' . $slidedeck['id'] . '" style="' . $styles_str . '">';
    foreach ( (array) $slides as $slide ) {
        $template_str.= '<dt>' . html_entity_decode( $slide['title'], ENT_QUOTES, 'UTF-8' ) . '</dt>';            // Slide Title Bar
        
        $template_str.= '<dd>' . do_shortcode( html_entity_decode( stripslashes( $slide['content'] ), ENT_QUOTES, 'UTF-8' ) ) . '</dd>';        // Slide Content
        
        $inc++;
    }
    $template_str .= '</dl>';
    return $template_str;
}


/**
 * Add a SlideDeck sidebar block to the post and page views
 * 
 * @uses add_meta_box()
 * @uses slidedeck_custom_box()
 */
function slidedeck_add_custom_box() {
    if ( function_exists( 'add_meta_box' ) ) {
        add_meta_box( 'slidedeck-sidebar', 'SlideDeck', 'slidedeck_cusom_box', 'post', 'side', 'high' );
        add_meta_box( 'slidedeck-sidebar', 'SlideDeck', 'slidedeck_cusom_box', 'page', 'side', 'high' );
    }
}
// Include sidebar.php view for post/page sidebar
function slidedeck_cusom_box() {
    global $post;
    
    $slidedeck_post_meta = array(
        '_slidedeck_slide_title' => get_the_title(),
        '_slidedeck_post_featured' => 0
    );
    
    foreach ( $slidedeck_post_meta as $meta_key => $meta_val ) {
        
        $post_meta_value = get_post_meta( $post->ID, $meta_key, true );
        if( !empty( $post_meta_value ) ) {
            $slidedeck_post_meta[$meta_key] = $post_meta_value;
        }
    }
    
    include( slidedeck_dir( '/views/sidebar.php' ) );
}
// Save dynamic SlideDeck data for this post
function slidedeck_save_dynamic_meta() {
    if( isset( $_POST['slidedeck-for-wordpress-dynamic-meta_wpnonce'] ) && !empty( $_POST['slidedeck-for-wordpress-dynamic-meta_wpnonce'] ) ) {
        if( !wp_verify_nonce( $_POST['slidedeck-for-wordpress-dynamic-meta_wpnonce'], 'slidedeck-for-wordpress' ) ) {
            return false;
        }
    
        $slidedeck_post_meta = array( '_slidedeck_slide_title', '_slidedeck_post_featured' );
        
        foreach( $slidedeck_post_meta as $meta_key ) {
            if( isset( $_POST[$meta_key] ) && !empty( $_POST[$meta_key] ) ) {
                update_post_meta( $_POST['ID'], $meta_key, $_POST[$meta_key] );
            } else {
                delete_post_meta( $_POST['ID'], $meta_key );
            }
        }
    }
}



add_action( 'posts_orderby', 'slidedeck_dynamic_orderby_popular' );
function slidedeck_dynamic_orderby_popular( $arg ) {
    global $wpdb, $slidedeck_orderby_popular_flag;
    
    if( $slidedeck_orderby_popular_flag === true ) {
        $arg = "$wpdb->posts.comment_count DESC, $wpdb->posts.post_date DESC";
    }
    
    return $arg;
}
add_action( 'parse_query', 'slidedeck_set_orderby_popular_flag' );
function slidedeck_set_orderby_popular_flag( $query ) {
    global $slidedeck_orderby_popular_flag;
    
    $slidedeck_orderby_popular_flag = false;
    
    if( isset( $query->query_vars['orderby'] ) ) {
        if( $query->query_vars['orderby'] == '_slidedeck_popularity' ) {
            $slidedeck_orderby_popular_flag = true;
        }
    }
}


function slidedeck_preview() {
    $slidedeck_id = $_GET['slidedeck_id'];
    $width = $_GET['width'];
    $height = $_GET['height'];
    $preview_w = $_GET['preview_w'];
    $preview_h = $_GET['preview_h'];

    $slidedeck = slidedeck_load( $slidedeck_id );
    
    $skin = slidedeck_get_skin( $slidedeck['skin'] );
    
    $dynamic = (boolean) $slidedeck['dynamic'];

    include( slidedeck_dir( 'views/preview.php' ) );
    
    slidedeck_print_footer_scripts();
    
    die();
}


function slidedeck_add_slide() {
    $count = $_GET['count'] + 1;
    $slide = array(
        'title' => "Slide " . $count,
        'slide_order' => $count,
        'content' => "",
        'gallery_id' => $_GET['gallery_id']
    );
    
    include( slidedeck_dir( 'views/_edit-slide.php' ) );
    
    die();
}


if ( function_exists( 'add_action' ) ) {
    // Add navigation elements to add SlideDecks to Post/Page editing views
    add_action( 'admin_menu', 'slidedeck_add_custom_box' );
    add_action( 'admin_menu', 'slidedeck_menu' );
    add_action( 'save_post', 'slidedeck_save_dynamic_meta' );
    
    // Add SlideDeck button to TinyMCE navigation
    add_action( 'admin_init', 'slidedeck_addbuttons' );
    add_action( 'admin_footer', 'slidedeck_tinymce_plugin_dialog' );
    
    // Add admin menu items to main control panel navigation
    add_action( 'admin_init', 'slidedeck_admin_init' );

    // Add JavaScript and Stylesheets for admin interface on appropriate pages
    add_action( 'admin_print_scripts-slidedeck_page_slidedeck/slidedeck_add_new', 'slidedeck_admin_head' );
    add_action( 'admin_print_scripts-slidedeck_page_slidedeck/slidedeck_dynamic', 'slidedeck_admin_head' );
    add_action( 'admin_print_scripts-toplevel_page_slidedeck', 'slidedeck_admin_head' );

    // Add required JavaScript and Stylesheets for displaying SlideDecks in public view    
    add_action( 'wp_print_scripts', 'slidedeck_print_scripts' );
    add_action( 'wp_print_styles', 'slidedeck_print_styles' );

    // Add shortcode to replace SlideDeck shortcodes in content with SlideDeck contents
    add_shortcode( 'SlideDeck', 'slidedeck_shortcode' );
    
    add_action( 'wp_ajax_slidedeck_preview', 'slidedeck_preview' );
    add_action( 'wp_ajax_slidedeck_add_slide', 'slidedeck_add_slide' );

    // Append necessary skin and initialization script commands to the bottom of the DOM for proper loading
    if( function_exists( 'wp_print_footer_scripts' ) ) {
        add_action( 'wp_print_footer_scripts', 'slidedeck_print_footer_scripts' );
    } else {
        add_action( 'wp_footer', 'slidedeck_print_footer_scripts' );
    }

    add_filter( 'tiny_mce_before_init', 'slidedeck_change_mce_options' );
    
    if( !is_admin() ) {
        add_action( 'wp', 'slidedeck_wp_hook' );
    }
}
